// **********************************************************************************
//  Direct3D 9.0 with Allegro Demo Project
//  by Homer Cuevas
// **********************************************************************************


#define ALLEGRO_NO_MAGIC_MAIN

#include <allegro.h>
#include <winalleg.h>
#include <d3d9.h>
#include <d3dx9.h>

#pragma comment(lib, "alleg.lib")
#pragma comment(lib, "d3dx9.lib")
#pragma comment(lib, "d3d9.lib")


//Allegro Timer Function

DWORD g_nCounter = 0;

void my_timer_handler()	{
	g_nCounter++;
}

// DirectX Stuff
LPDIRECT3D9 g_pD3D = NULL; // Used to create the D3DDevice
LPDIRECT3DDEVICE9 g_pd3dDevice = NULL; // Our rendering device
LPD3DXMESH g_pMesh = NULL; // Our mesh object in sysmem
D3DMATERIAL9 *g_pMeshMaterials = NULL; // Materials for our mesh
LPDIRECT3DTEXTURE9 *g_pMeshTextures  = NULL; // Textures for our mesh
DWORD g_dwNumMaterials = 0L;   // Number of mesh materials

// Core Functions
void SetupMatrices(void);
void EndDirectX(void);
int StartAllegro(void);
int StartDirectX(void);
// Mesh Functions
int LoadX(char *szFile);
void RenderX(void);

int WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT)	{
	if (StartAllegro())
		return 1;
	
	if (StartDirectX())
		return 1;

	if (LoadX("tiger.x"))
		return 1;

	while(!keypressed())	{

		SetupMatrices();

		// Clear the backbuffer and the zbuffer
		g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 
							 D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );
    
		// Begin the scene
		if( SUCCEEDED(g_pd3dDevice->BeginScene()))	{

			RenderX();

			// End the scene
			g_pd3dDevice->EndScene();
		}

		// Present the backbuffer contents to the display
		g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
	}

	EndDirectX();

	return 0;
}

int LoadX(char *szFile)	{
	LPD3DXBUFFER pD3DXMtrlBuffer;
	// load the mesh
	 if( FAILED( D3DXLoadMeshFromX(szFile, D3DXMESH_SYSTEMMEM, 
                                   g_pd3dDevice, NULL, 
                                   &pD3DXMtrlBuffer, NULL, &g_dwNumMaterials, 
                                   &g_pMesh ) ) )	{
		 MessageBox(win_get_window(),"Error Loading Mesh","Mesh Error",MB_OK);
		 return 1;
	 }

    D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)pD3DXMtrlBuffer->GetBufferPointer();
    g_pMeshMaterials = new D3DMATERIAL9[g_dwNumMaterials];
    if( g_pMeshMaterials == NULL )
        return 1;
    g_pMeshTextures  = new LPDIRECT3DTEXTURE9[g_dwNumMaterials];
    if( g_pMeshTextures == NULL )
        return 1;

    for(DWORD i=0; i<g_dwNumMaterials; i++ )    {
        // Copy the material
        g_pMeshMaterials[i] = d3dxMaterials[i].MatD3D;

        // Set the ambient color for the material (D3DX does not do this)
        g_pMeshMaterials[i].Ambient = g_pMeshMaterials[i].Diffuse;

        g_pMeshTextures[i] = NULL;
        if( d3dxMaterials[i].pTextureFilename != NULL && 
			lstrlen(d3dxMaterials[i].pTextureFilename) > 0 )	{

			if (FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, 
                                                d3dxMaterials[i].pTextureFilename, 
                                                &g_pMeshTextures[i] ) ) )    {
					MessageBox(win_get_window(),"Error Loading Texture for Mesh","Texture Error",MB_OK);
					return 1;
			 }
		}
	}
			
	pD3DXMtrlBuffer->Release();

	return 0;
}

void SetupMatrices(void)	{
	// Setup the world, view, and projection matrices

	// Set up world matrix
	D3DXMATRIXA16 matWorld;
	D3DXMatrixRotationY( &matWorld, g_nCounter/1000.0f );
	g_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );

	// Set up our view matrix. A view matrix can be defined given an eye point,
	// a point to lookat, and a direction for which way is up. Here, we set the
	// eye five units back along the z-axis and up three units, look at the 
	// origin, and define "up" to be in the y-direction.
	D3DXVECTOR3 vEyePt( 0.0f, 3.0f,-5.0f );
	D3DXVECTOR3 vLookatPt( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 vUpVec( 0.0f, 1.0f, 0.0f );
	D3DXMATRIXA16 matView;
	D3DXMatrixLookAtLH( &matView, &vEyePt, &vLookatPt, &vUpVec );
	g_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

	// For the projection matrix, we set up a perspective transform (which
	// transforms geometry from 3D view space to 2D viewport space, with
	// a perspective divide making objects smaller in the distance). To build
	// a perpsective transform, we need the field of view (1/4 pi is common),
	// the aspect ratio, and the near and far clipping planes (which define at
	// what distances geometry should be no longer be rendered).
	D3DXMATRIXA16 matProj;
	D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, 1.0f, 1.0f, 100.0f );
	g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );
}

void EndDirectX(void)	{
	// Free all resources	

    if( g_pMeshMaterials != NULL ) 
        delete[] g_pMeshMaterials;

    if( g_pMeshTextures )    {
        for( DWORD i = 0; i < g_dwNumMaterials; i++ )	{
            if( g_pMeshTextures[i] )
                g_pMeshTextures[i]->Release();
        }
        delete[] g_pMeshTextures;
    }
    if( g_pMesh != NULL )
        g_pMesh->Release();
    
    if( g_pd3dDevice != NULL )
        g_pd3dDevice->Release();

    if( g_pD3D != NULL )
        g_pD3D->Release();
}

int StartDirectX(void)	{
	D3DPRESENT_PARAMETERS d3dpp;

	//creating direct3D object
	g_pD3D = Direct3DCreate9(D3D_SDK_VERSION);
	if	(g_pD3D == NULL)	{
		MessageBox(win_get_window(),"Error Creating Direct3D Object","D3D Object Error",MB_OK);
		return 1;
	}

	//setting parameters that will be needed to created direct3D device
	ZeroMemory(&d3dpp, sizeof(d3dpp));			
	d3dpp.Windowed = false;						
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.EnableAutoDepthStencil = true;					
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;		
	d3dpp.hDeviceWindow = win_get_window();
	d3dpp.BackBufferWidth = 800;					
	d3dpp.BackBufferHeight = 600;					
	d3dpp.BackBufferFormat = D3DFMT_R5G6B5;					
	d3dpp.MultiSampleType = D3DMULTISAMPLE_NONE;

	//creating direct3D device
	if(FAILED(g_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, win_get_window(), D3DCREATE_SOFTWARE_VERTEXPROCESSING,&d3dpp, &g_pd3dDevice)))	{
		MessageBox(win_get_window(),"Error Creating Direct3D Device","D3D Device Error",MB_OK);
		return 1;
	}

    // Turn on the zbuffer
    g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );

    // Turn on ambient lighting 
    g_pd3dDevice->SetRenderState( D3DRS_AMBIENT, 0xffffffff );

	return 0;
}

int StartAllegro(void)	{
	allegro_init();

	if (set_gfx_mode(GFX_DIRECTX_WIN, 800, 600, 0, 0) < 0) {
		MessageBox(win_get_window(), allegro_error, "Error setting video mode", MB_OK);
		return 1;
	}

	install_timer();
	install_keyboard();
	install_mouse();
	show_mouse(NULL);

	install_int(my_timer_handler, 1);

	return 0;
}

void RenderX(void)	{
	// Meshes are divided into subsets, one for each material. Render them in
	// a loop
	for( DWORD i=0; i<g_dwNumMaterials; i++ )	{
		// Set the material and texture for this subset
		g_pd3dDevice->SetMaterial(&g_pMeshMaterials[i]);
		g_pd3dDevice->SetTexture(0, g_pMeshTextures[i]);

		// Draw the mesh subset
		g_pMesh->DrawSubset( i );
	}
}